# 1.1 データの準備
library(MASS)
?birthwt
d <- birthwt
d <- d[,-c(1,4,7:9)]
head(d)
d$lwt <- 0.453592*d$lwt
table(d$ptl)
d$ptl <- ifelse(d$ptl>=1,1,0)
summary(d)

# 1.2 等分散性を仮定したt検定
boxplot(d$bwt)
boxplot(bwt~smoke,d,notch=T)
res <- t.test(bwt~smoke,d,var.equal=T)
print(res)
names(res)
(b <- res$estimate)
b[2]-b[1]

# 1.3 ダミー変数による回帰
plot(bwt~smoke,d)
res <- lm(bwt~smoke,d)
abline(res,col="red")
summary(res)

# 1.4 連続変数による回帰
plot(bwt~lwt,d)
res <- lm(bwt~lwt,d)
abline(res,col="red")
summary(res)

# 1.5 相関係数と標準化データの回帰
x <- d$lwt
y <- d$bwt
cor(x,y)
zx <- (x-mean(x))/sd(x)
head(cbind(x,zx))
mean(zx)
sd(zx)
zy <- (y-mean(y))/sd(y)
plot(zx,zy)
res <- lm(zy~zx)
abline(res,col="red")
round(summary(res)$coefficients,3)

# 1.6 重回帰モデル
res <- lm(bwt~lwt+age+ptl+smoke,d)
summary(res)

# 1.7 モデルの適合度
names(res)
names(summary(res))
fit <- res$fitted.values
head(cbind(d,fit))
obs <- d$bwt
(sum((obs-fit)^2)/(nrow(d)-5))^0.5
myr <- range(c(fit,obs))
plot(fit,obs,xlim=myr,ylim=myr)
abline(a=0,b=1,col="red")
cor(fit,obs)^2

# 1.8 交差検証法による予測残差標準偏差
s <- 0
for(i in 1:nrow(d)){
  res <- lm(bwt~lwt+age+ptl+smoke,d[-i,]) # i番目のデータを外して推定
  pre <- predict(res,d[i,]) # i番目のデータで予測
  s <- s+(obs[i]-pre)^2
}
s <- s/nrow(d)
print(s^0.5)

# 1.9 交互作用項
res <- lm(bwt~lwt+age+ptl+smoke+ptl:smoke,d)
summary(res)

# 1.10 パス解析で重回帰モデル
library(lavaan)
res <- sem("bwt~lwt+age+ptl+smoke",d)
parameterEstimates(res,ci=F,remove.nonfree=T)
library(semPlot)
semPaths(res,"model","est",style="lisrel")

# 1.11 パス解析
model <- "
bwt~lwt+ptl+smoke
ptl~smoke"
res <- sem(model,d)
parameterEstimates(res,ci=F,remove.nonfree=T)
semPaths(res,"model","est",style="lisrel")

# 1.12 課題
d <- ToothGrowth
head(d)
aggregate(len~dose+supp,d,mean)
