#2.1. データの準備
d <- read.csv("lung50.csv")
head(d)

#2.2．クロス集計表（分割表）
o <- xtabs(~smoke+lung, data=d)
o
f <- addmargins(o)
f

#2.3．期待度数と検定統計量
n <- f[3,3]
e <- 0*o
for(i in 1:2)for(j in 1:2) e[i,j] <- n*(f[i,3]/n)*(f[3,j]/n)
# 繰り返し範囲を{}できちんと書く場合
# for(i in 1:2){
# 	for(j in 1:2){
# 	e[i,j] <- n*(f[i,3]/n)*(f[3,j]/n)
# 	}
# }
e
(o-e)/sqrt(e)
(o-e)^2/e
chisq <- sum((o-e)^2/e)
chisq

#2.4．ピアソンのカイ2乗検定
qchisq(0.95,df=1)  # qchisq(0.05,df=1,lower.tail=F) と同じ
qchisq(0.99,df=1)
chisq.test(o,correct=FALSE)
1-pchisq(5.4329,df=1)

#2.5．相関係数との関係
(r <- cor(d)[1,2])
n*r^2

#2.6．オッズ比
p <- o[,2]/(o[,1]+o[,2])
p0 <- p[1] # 非喫煙群
p1 <- p[2] # 喫煙群
p1/p0
odds0 <- p0/(1-p0)
odds1 <- p1/(1-p1)
(or <- odds1/odds0)
(or <- o[1,1]*o[2,2]/(o[1,2]*o[2,1]))

#2.7．ロジスティック回帰
e1 <- runif(50,min=-0.02,max=0.02)
e2 <- runif(50,min=-0.02,max=0.02)
plot(d$smoke+e1,d$lung+e2,xlab="smoke",ylab="lung")
(res_lm <- lm(lung~smoke, data=d))
abline(res_lm,col="red",lwd=3)
res <- glm(lung~smoke, data=d, family="binomial")
ff <- summary(res)$coefficients
summary(res)
exp(ff[2,1])

#2.8．用量反応曲線
d <- read.csv("dosebinary.csv")
head(d)
dim(d)
xtabs(~dose+dead,data=d)
plot(d$dose,d$dead)

e1 <- runif(1119,min=-0.05,max=0.05) 
plot(d$dose,d$dead+e1)
res <- glm(dead~dose, data=d, family="binomial")
summary(res)
exp(31.224*0.02)
exp(31.224*0.04)

(bvec <- res$coefficients)
(doses <- seq(from=1.5, to=2, by=0.01))
z <- bvec[1]+bvec[2]*doses
p <- exp(z)/(1+exp(z))
plot(d$dose,d$dead,xlim=range(doses))
lines(doses,p,col="red")  # points(doses,p,col="red",type=”l”)

f <- xtabs(~dose+dead,data=d)
phat <- f[,2]/(f[,1] + f[,2])
ddose <- as.numeric(rownames(f))
plot(ddose,phat,type="b")
lines(doses,p,col="red")

ED50 <- -bvec[1]/bvec[2]
print(ED50)
abline(h=0.5,col="green")
abline(v=ED50,col="green")

#2.9．グループデータのロジスティック回帰
f <- xtabs(~dose+dead,d)
dd <- data.frame(dose=as.numeric(rownames(f)),dead=f[,2],alive=f[,1])
rownames(dd) <- 1:nrow(dd)
dd
res <- glm(cbind(dead, alive)~dose,data=dd,family="binomial")
summary(res)

#2.10. 課題
library(MASS)
d <- birthwt
d <- d[,-c(4,7:10)]
d$lwt <- 0.453592*d$lwt
d$ptl <- ifelse(d$ptl>=1,1,0)
summary(d)
