# 3.1 ElemStatLearn パッケージの利用
# 3.1.1 ElemStatLearn パッケージを利用する場合
urlst <- "https://cran.r-project.org/src/contrib/Archive/ElemStatLearn/"
filest <- "ElemStatLearn_2015.6.26.2.tar.gz"
install.packages(paste0(urlst,filest),repos=NULL,type="source")
library(ElemStatLearn)
data(bone)
?bone
d0 <- bone

# 3.1.2 ElemStatLearn パッケージを利用しない場合
d0 <- read.csv("ElemStatLearn_bone.csv")
head(d0)
d0 <- subset(d0,gender=="male") # 男性だけ抽出する
d0 <- d0[order(d0$age),] # 年齢でソートする
x <- d0$age # 年齢をxとする
y <- d0$spnbmd # 骨密度の相対変化をyとする
d <- data.frame(x,y) # xとyのデータフレームをdとする
dim(d)
head(d)
plot(x,y)
(res <- lm(y~x))
abline(res,col="red")
abline(h=0,col="blue")

# 3.4 重みの準備
x0 <- 12
w <- dnorm(x,mean=x0,sd=1.1)
w <- w/sum(w) #基準化
plot(x,w,type="h")

# 3.5 重み付き回帰
myalpha <- w/max(w) #最大値が1になるように変換
plot(x,y,pch=19,col=rgb(1,0,0,alpha=myalpha))
points(x,y,pch=21)
(res <- lm(y~x,weight=w))
abline(res,col="blue")

# 3.6 局所線形回帰
plot(x,y)
fitted <- 0*y
rss <- 0 # 残差平方和
for(i in 1:nrow(d)){
  d$w <- dnorm(x,mean=x[i],sd=1.1) # i 番目年齢による重み
  res <- lm(y~x,weight=w,d) # 直線回帰
  fitted[i] <- predict(res,d[i,]) # 予測
  rss <- rss+(y[i]-fitted[i])^2 # 残差平方
}
lines(x,fitted,col="red",lwd=3)
print(rss)
res <- loess(y~x)
fitted.loess <- res$fitted
lines(x,fitted.loess,col="blue",lwd=3)

# 3.7 多項式回帰
plot(x,y)
(res <- lm(y~x+x^2+x^3)) # 直線と同じになる
(res <- lm(y~x+I(x^2)+I(x^3))) # 3次曲線
lines(x,res$fitted.values,col="red",lwd=3)
b <- res$coefficients
xs <- seq(from=0,to=25,by=1)
plot(0,0,type="n",
     xlim=c(0,25),ylim=c(-5,5),xlab="x",ylab="")
for(j in 1:4)lines(xs,b[j]*xs^(j-1),col=j,lwd=3)
legendst <- paste0(round(b,4),"x^",1:4-1)
legend("topleft",lwd=3,col=1:4,legend=legendst)

# 3.8 折れ線のあてはめ
sp <- function(k) ifelse(x-k>0,x-k,0)
plot(x,sp(14),type="l",col="red",lwd=3)
(res <- lm(y~x+sp(14)))
plot(x,y)
lines(x,res$fitted.value,col="red",lwd=3)

# 3.9 スプライン基底関数による重回帰
ks <- seq(from=10, to=24, by=2)
sps <- x
for(k in ks) sps <- cbind(sps, sp(k))
colnames(sps) <- c("x",paste0("sp",ks))
sps <- as.data.frame(sps)
head(sps)
plot(x,y)
(res <- lm(y~.,sps))
lines(x,res$fitted.value,col="red",lwd=3)
rug(ks,col="blue")
best <- step(res, trace=T)
print(best)
lines(x,best$fitted.value,col="orange",lwd=3)
lines(x,fitted.loess,col="green",lwd=3)

# 3.10 ロバスト回帰
cars2 <- cars
cars2[1:4,2] <- 120
head(cbind(cars,cars2))
plot(cars2)
res <- lm(dist~speed, cars2)
abline(res,col="red")
for(i in 1:3){
  res <- lm(dist~speed, cars2,
    weights=1/abs(res$residuals))
  abline(res,col="blue")
}
library(MASS)
res <- rlm(dist~speed, cars2)
abline(res,col="green")

# 3.11 課題
(d <- Theoph[Theoph$Subject==1,])
