# 4.1. データの準備
library(ade4)
data(olympic)
?olympic
d <- olympic$tab
head(d,3)
colnames(d)[c(3,4,7,8)] <-c("shot","high","disc","pole")
round(t(apply(d,2,summary)),1)
index <- c(1,5,6,10) # 走る記録を負にする
d[,index] <- -d[,index]
head(d,1)

# 4.2 散布図と直線回帰
pairs(d,col="red",pch=19,cex=0.7)
plot(d$shot,d$disc,xlab="shot",ylab="disc",cex=1.5)
(res <- lm(disc~shot,d))
abline(res,col="red")
cor(d$shot,d$disc)

# 4.3 相関行列と因子分析
corx <- cor(d)
round(corx,3)
myk <- 3
res <- factanal(covmat=corx, factors=myk)
res$loadings

# 4.4 相関係数と距離
z <- scale(d)
round(head(z,3),3)
apply(z,2,mean)
apply(z,2,sd)
distz <- dist(t(z))
round(as.matrix(distz),3)
plot(as.matrix(corx),as.matrix(distz)^2)
n <- nrow(z)
abline(a=2*(n-1),b=-2*(n-1),col="red")

# 4.5 階層型クラスター分析
res <- hclust(distz)
plot(res)
myk <- 3
resh <- rect.hclust(res,k=myk,border=1:myk+1)
resh

# 4.6 多次元尺度法と主成分分析
# 4.6.1 多次元尺度法
res <- cmdscale(distz)
mylim <- range(res[,1:2])
plot(res,type="n",xlim=mylim,ylim=mylim,main="MDS")
text(res,rownames(res))
distz2 <- dist(res)
plot(distz,distz2,xlab="D33",ylab="D2")
cor(as.vector(distz),as.vector(distz2))

# 4.6.2 主成分分析
res <- prcomp(t(z))
mylim <- range(res$x[,1:2])
plot(res$x[,1:2],type="n",xlim=mylim,ylim=mylim,main="PCA")
text(res$x[,1:2],rownames(t(z)))

# 4.7 隣接行列と無向グラフ
(cutoff <- quantile(distz,0.25))
distz01 <- 1*(as.matrix(distz)<=cutoff)
diag(distz01) <- 0
distz01
library(igraph)
g <- graph.adjacency(distz01,mode="undirected")
# 初期値が同じでも配置が若干異なることがあります．
set.seed(123)
plot(g)

# 4.8 次元縮約としての主成分分析
# 4.8.1 2次元のデータを1 次元に縮約する
z37 <- z[,c(3,7)]
myr <- range(z37)
plot(z37,type="n",xlim=myr,ylim=myr)
text(z37,rownames(z37))
deg <- 45 # 軸の向きを X 軸に対する角度で与える
theta <- pi*deg/180
plot(z37,xlim=myr,ylim=myr,col="blue")
# 軸の向き
(r <- c(cos(theta),sin(theta)))
arrows(x0=0,y0=0,x1=r[1],y1=r[2],col="red",lwd=3)
# 垂線を下す軸
abline(a=0,b=r[2]/r[1],col="red") 
# 主成分軸と主成分得点
PC1 <- as.vector(z37 %*% r) 
for(i in 1:nrow(z37)){
  arrows(x0=z37[i,1],y0=z37[i,2],
    x1=PC1[i]*r[1],y1=PC1[i]*r[2],length=0.1)
}
var(PC1)   # 主成分の情報量（分散）
var(PC1)/2 # 寄与率
res <- prcomp(z37)
res$rotation # 軸の向き
head(res$x) # 主成分得点
myr <- range(res$x)
plot(res$x,type="n",xlim=myr,ylim=myr)
abline(h=0,col=2)
abline(v=0,col=2)
text(res$x,rownames(z37))
summary(res)$importance
cor(res$x[,1],z37)
sd(res$x[,1])*res$rotation[,1]

# 4.8.2 10次元のデータを 2 次元に縮約する
res <- prcomp(z)
plot(res$x[,1:2],type="n",main="PCA")
text(res$x[,1:2],rownames(z))
round(summary(res)$importance,3)
barplot(summary(res)$importance[3,],main="Cumulative Proportion")
round(res$rotation[,1:2],2)
colSums(res$rotation^2)
mybarplot <- function(x,mymain){
    mycol <- ifelse(x>0,2,4)
    barplot(x,col=mycol,main=mymain)
  }
mybarplot(res$rotation[,1],"PC1")
mybarplot(res$rotation[,2],"PC2")
biplot(res,scale=F)
drank <- apply(-d,2,rank)
id <- 17
drank[id,]
res$x[id,1:2]
R12 <- res$rotation[,1:2]
index <- upper.tri(corx)
cosR12 <- 0*corx
p <- nrow(R12)
for(i in 1:(p-1))for(j in (i+1):p)
  cosR12[i,j] <- sum(R12[i,]*R12[j,])/
    (sum(R12[i,]^2)*sum(R12[j,]^2))^0.5 # 内積から cos を求める
acosR12 <- acos(cosR12)*(180/(2*pi)) #  cos の値から角度に変換
plot(as.vector(corx[index]),as.vector(acosR12[index]),col=2,
     xlab="相関係数",ylab="係数ベクトルの角度")

# 4.9 課題
library(mlbench)
data(PimaIndiansDiabetes2)
index <- complete.cases(PimaIndiansDiabetes2)
d <- PimaIndiansDiabetes2[index,]
y <- ifelse(d$diabetes=="pos",1,0)
x <- d[,-9]
res <- glm(y~.,x,family="binomial")
summary(res)
