# 5.1. タイタニックデータの読み込みと対照群の設定
d <- read.csv("titanic.csv",stringsAsFactors=T)
head(d)
summary(d)
head(d$Age)
levels(d$Age)
d$Age <- relevel(d$Age,ref="Child") 
levels(d$Age)

# 5.2. クロス集計表, 多次元分割表
xtabs(~Class,d)
xtabs(~Class+Died,d)
f <- xtabs(~Class+Sex+Age+Died,d)
ftable(f)

# 5.3. ロジスティック回帰, 一般化線形モデル
res <- glm(Died~Class+Sex+Age,family="binomial",d)
glm(Died~.,family="binomial",d)
summary(res)
res$coefficients
exp(res$coefficients)
round(exp(res$coefficients),2)

# 5.4. ロジスティック判別
# 5.4.1　予測確率の計算と分布
res$fitted.values
d$Died==1
(x1 <- res$fitted.values[d$Died==1]) 
hist(x1,freq=F,breaks=5,ylim=c(0,3))
z <- (0:100)/100
lines(z,dnorm(z,mean(x1),sd(x1)),col="red")

plot(z,dnorm(z,mean(x1),sd(x1)),col="red",type="l",xlim=c(0,1),
     main="予測値分布",xlab="死亡確率",ylab="密度")
x0 <- res$fitted.values[d$Died==0]
lines(z,dnorm(z,mean(x0),sd(x0)),col="blue")
rug(res$fitted.values[d$Died==1],col="red")
rug(res$fitted.values[d$Died==0],col="blue",side=3)
legend("left",lty=c(1,1),col=c("red","blue"),
	legend=c("死亡: Died=1","生存: Died=0"),title="観測値")

# 5.4.2　判別ルールの設定とその評価
p <- 0.5 # 閾値を0.5として判別する
Predicted <- ifelse(res$fitted.values>=p,1,0)
f <- xtabs(~d$Died+Predicted)
print(f)
# 正分類率
r <- 100*sum(diag(f))/sum(f)
print(r)
#tp     感度 = 真陽性率 P(予測値=1|観測値=1)
#fp 1-特異度 = 1-真陰性率 = 偽陽性率 P(予測値=1|観測値=0)
tp <-   f[2,2]/(f[2,1]+f[2,2])
fp <- 1-f[1,1]/(f[1,1]+f[1,2])
tp; fp

# 5.5. 閾値の変更：感度と 1 − 特異度
(p <- 1:8/10)
tp <- c()
fp <- c()
for(j in 1:8){
  Predicted <- ifelse(res$fitted.values >= p[j],1,0)
  f <- xtabs(~d$Died+Predicted)
  tp[j] <- f[2,2]/(f[2,1]+f[2,2])
  fp[j] <- 1-f[1,1]/(f[1,1]+f[1,2])
}
cbind(p,fp,tp)
plot(fp,tp,type="l",xlim=c(0,1),ylim=c(0,1),
     xlab="偽陽性率=1-特異度",ylab="真陽性率=感度")
text(x=fp,y=tp,labels=p,col="red")

# 5.6. ROC曲線
library(pROC)
roc1 <- roc(d$Died~res$fitted.values)
names(roc1)
plot(1-roc1$specificities,roc1$sensitivities,type="l",
  col="red",xlab="偽陽性率=1-特異度",ylab="真陽性率=感度")
lines(fp,tp,col="blue")
roc1$auc

# 5.7. CART, 分類木, 決定木
library(rpart)
res <- rpart(as.factor(Died)~., data=d,method="class")
library(partykit)
plot(as.party(res))
res

tree.Predicted <- predict(res)
c <- ifelse(tree.Predicted[,2] >=0.5, 1,0)
f <- table(d$Died, c) # xtabs(~d$Died +c)と同じ
print(f)
100*sum(diag(f))/sum(f)

# 5.8. 判別の性能比較
roc2 <- roc(d$Died~tree.Predicted[,2])
plot(1-roc1$specificities,roc1$sensitivities,type="l",
  col="red",xlab="偽陽性率=1-特異度",ylab="真陽性率=感度")
lines(1-roc2$specificities,roc2$sensitivities,col="blue")
abline(a=0,b=1,col="gray",lty=3)
legend("bottomright",lty=c(1,1),
  col=c("red","blue"),legend=c("logistic","cart"))
roc1$auc;roc2$auc

# 5.9. 課題
library(MASS)
d <- birthwt
d <- d[,-c(4,7:10)]
d$lwt <- 0.453592*d$lwt
d$ptl <- ifelse(d$ptl>=1,1,0)
