# 6.1	データの準備
dt <- read.csv("h18koe.csv", stringsAsFactors=FALSE)
colnames(dt)
mytext_field <- "意見.感想"
example <- 98
dt[example,mytext_field]

# 6.2	分かち書き
library(quanteda)
corp <- corpus(dt,text_field=mytext_field)
corp[example]
toks <- tokens(corp)
toks[example]

# 6.3 用例確認
kwic(toks,"欲しい",window=7)

# 6.4 単語の抽出と削除
toks <- tokens_select(toks, "^[０-９ぁ-んァ-ヶー一-龠]+$",
  valuetype ="regex", padding=T)
toks[example]
toks <- tokens_remove(toks,"^[ぁ-ん]+$",valuetype="regex",padding=T)
toks[example]
toks <- tokens_remove(toks,c("思い","思う","感じ"),
                      valuetype="fixed",padding=T)

# 6.5 複合語の選択
library(quanteda.textstats)
mymin_count <- 5
seqs <- textstat_collocations(toks,min_count=mymin_count)
print(seqs)
myz <- 3
seqs <- seqs[seqs$z>myz,]
toks <- tokens_compound(toks,seqs,concatenator='') 

# 6.6 単語や同義語の置換
toks <- tokens_replace(toks,
          c("美海水族館","海水族館","料理","食事"),
          c(rep("美ら海水族館",2),rep("沖縄料理",2)),
          valuetype="fixed")

# 6.7 文書単語行列
df <- dfm(toks)
dim(df)
df[1:6,1:10]
mymin_nchar <- 2
df <- dfm_select(df,min_nchar=mymin_nchar)
dim(df)
min_termfreq <- 6
df <- dfm_trim(df,min_termfreq=min_termfreq)
dim(df)

# 6.8 ワードクラウド
library(RColorBrewer)
display.brewer.all()
library(quanteda.textplots)
set.seed(123)
textplot_wordcloud(df,random_order=F,
                   rotation=0,
                   color = brewer.pal(8,"Dark2"),
                   min_size=0.5,max_size=6)

# 6.9 数値データへの変換
d <- as.matrix(df)
dim(d)
index <- order(colSums(d),decreasing=T) 
d <- d[,index]
d[1:6,1:6]
colSums(d)[1:20] # トップ20表示
d01 <- d
d01[d01>1] <- 1 
head(colSums(d01))

# 6.10 ジャッカード距離
f <- xtabs(~沖縄+観光,d01)
print(f)
dist(t(d01[,c("沖縄","観光")]),method="binary")
distz <-dist(t(d01),method="binary")
f <- as.matrix(distz)
round(f[1:6,1:6],3)

# 6.11 階層型クラスター分析
par(cex=0.8)
res <- hclust(distz,method="ward.D2")
plot(res)

# 6.12 多次元尺度法
res <- cmdscale(distz)
par(cex=1)
library(wordcloud)
set.seed(123)
textplot(res[,1],res[,2],words=rownames(res),show.lines=F,
         xlim=range(res[,1]),ylim=range(res[,2]))

# 6.13 隣接行列
threshold <- sort(as.matrix(distz))
p <- attr(distz,"Size")
threshold <- threshold[-(1:p)]
head(threshold)
m <- length(threshold)
nedge <- 0*(1:m)
for(i in 1:m) nedge[i] <- (sum(as.matrix(distz)<=threshold[i])-p)/2
plot(threshold,nedge,log="xy")
myedge <- 110
mythreshold <- threshold[sum(nedge <= myedge)]
distz01 <- 1*(as.matrix(distz)<=mythreshold)
diag(distz01) <- 0
n <- colSums(distz01)
distz01 <- distz01[n>0,n>0]

# 6.14 ネットワーク分析
library(igraph)
g <- graph.adjacency(distz01,mode="undirected")
set.seed(123)
plot.igraph(g,vertex.color="white")
mydeg <- degree(g)
head(mydeg)
mycom <- edge.betweenness.community(g) 
mycom$membership
mycol <- c(brewer.pal(12,"Set3"),
           brewer.pal(8,"Pastel2"),
           brewer.pal(9,"Pastel1"))
head(mycol)
# 初期値が同じでも配置が若干異なることがあります．
set.seed(123) 
plot.igraph(g,
  vertex.color=mycol[mycom$membership],
  vertex.frame.color=NA,
  vertex.label.color="black",
  vertex.size=10+15*mydeg/max(mydeg),
  layout= layout.fruchterman.reingold)

# 6.15 対応分析
library(makedummies)
dt$年代 <- as.factor(dt$年代)
table(dt$年代,useNA="always")
x <- makedummies(dt,basal_level=T,col="年代")   
head(x)
x[is.na(x)] <- 0
f <- t(as.matrix(d01)) %*% x
head(f)
library(MASS)
res <- corresp(f,nf=2)
xy <- rbind(res$rscore,res$cscore)
mycol <- c(rep(1,nrow(res$rscore)),
           rep(2,nrow(res$cscore)))
par(cex=0.8)
set.seed(123)
textplot(xy[,1],xy[,2],words=rownames(xy),
         col=mycol,show.lines=F,
         xlim=range(xy[,1]),ylim=range(xy[,2]))

# 6.16 課題
dt <- read.csv("wiki.csv",stringsAsFactors=FALSE)
colnames(dt)
mytext_field <- "description" # テキスト列
myz <- 3 # 固定
mymin_nchar <- 2 # 固定
mymin_count <- 3 # 複合語の最小頻度
min_termfreq <- 3 # 頻出語の最小頻度
library(quanteda)
corp <- corpus(dt,text_field=mytext_field)
toks <- tokens(corp)
toks <- tokens_select(toks, "^[０-９ぁ-んァ-ヶー一-龠]+$",
  valuetype ="regex", padding=T)
toks <- tokens_remove(toks,"^[ぁ-ん]+$",valuetype="regex",padding=T)
library(quanteda.textstats)
seqs <- textstat_collocations(toks,min_count=mymin_count)
seqs <- seqs[seqs$z>myz,]
toks <- tokens_compound(toks,seqs,concatenator='') 
df <- dfm(toks)
df <- dfm_select(df,min_nchar=mymin_nchar)
df <- dfm_trim(df,min_termfreq=min_termfreq)
d <- as.matrix(df)
rownames(d) <- dt$keyword
index <- order(colSums(d),decreasing=T) 
d <- d[,index] # のべ出現回数
d[1:6,1:6]
d01 <- d
d01[d01>1] <- 1 # 単語の出現の有無
dim(d01)
