# 7.1. 1次元データのk-means法による分類
?iris
head(iris)
table(iris$Species)
d <- sort(iris$Petal.Width)
hist(d)
myk <- 2
set.seed(1)
res <- kmeans(d, centers=myk)
res$centers
table(myc <- res$cluster)
rug(res$centers,col=3,lwd=3)
rug(jitter(d[myc==1]),col=2) 
rug(jitter(d[myc==2]),col=4) 

# 7.2　1次元データの混合正規分布による分類
library(mclust)
myk <- 2
set.seed(1)
res <- densityMclust(d,G=myk,modelNames="V")
res$parameters$mean
res$parameters$variance$sigmasq^0.5
plot(res, what="density",data=d,col=2)
rug(jitter(d),col=4) 
head(unique(cbind(d,res$z))) # 所属確率
colSums(res$z) # 所属確率の和
sum(colSums(res$z)) # データ数
colSums(res$z)/sum(colSums(res$z)) #その比率が混合率
res$parameters$pro #混合率
plot(res, what="density")
table(myc <- res$classification)
rug(jitter(d[myc==1]),col=2) 
rug(jitter(d[myc==2]),col=4) 

# 7.3 2次元データのk-means法による分類
d <- subset(iris,select=c(Sepal.Length,Sepal.Width))
d <- scale(d)
plot(d)
centers0 <- cbind(c(-1,2),c(1,0)) # 初期値その1
#centers0 <- cbind(c(0.2,-0.2),c(0.7,-0.7)) # 初期値その2
res <- kmeans(d,centers=centers0)
res$tot.withinss
plot(d,col=res$cluster+1,pch=19)
points(centers0,pch=2,cex=3)
points(res$centers,pch=17,cex=3)
legend("topright", pch=c(2,17),legend=c("初期値","最適値"),cex=1.5)

# 7.4　2次元データの混合正規分布による分類
myk <- 2
set.seed(1)
res <- densityMclust(d,G=myk,modelNames="VVV") 
res$parameters$mean
table(res$classification)
plot(res, what="density")
points(d,col=res$classification+1,pch=19)
plot(res, what="density",type="persp",phi=20,theta=-30,col=3)

# 7.5 経時測定データに対する混合正規分布の応用
library(nlme)
?Orthodont
Orthodont$distance[Orthodont$Subject=="M13"]
d <- subset(Orthodont,Subject!="M13") #M13を除外
head(d)
d$age <- d$age-8
boxplot(distance~Sex+age,data=d,las=3,col=c(4,2),xlab="")
# 個体ごとの経時変化を直線で要約
plot(d$age,d$distance,type="n")
(index <- unique(d$Subject))
bmat <- matrix(nrow=length(index),ncol=2)
rownames(bmat) <- index
colnames(bmat) <- c("intercept","gradient")
library(MASS)
for(i in 1:length(index)){
  di <- subset(d,Subject==index[i])  
  mycol <- ifelse(di$Sex[1]=="Male",4,2)
  lines(di$age,di$distance,col=mycol) #性別で色分け
  res <- rlm(distance~age,di,maxit=100)
  bmat[i,] <- res$coefficients
}
head(bmat)
# BICによるコンポーネント数の選択
BIC <- mclustBIC(bmat,G=1:3,modelNames="VVV")
plot(BIC)
# コンポーネント数3の混合正規分布
myk <- 3
set.seed(1)
res2 <- densityMclust(bmat,G=myk,modelNames="VVV")
plot(res2, what="density",type="hdr")
myc <- res2$classification
points(bmat,col=myc+1,pch=19,cex=1)
text(bmat,labels=rownames(bmat),pos=1)
(means <- t(res2$parameters$mean))
for(k in 1:myk)
  points(means[k,1],means[k,2],col=k+1,pch=17,cex=2)
legend("topleft",legend=paste0("コンポーネント",1:myk),col=1:myk+1,lwd=5)
# 混合正規分布による折れ線の分類結果
plot(d$age,d$distance,type="n")
mycol <- res2$classification+1
for(i in 1:length(index)){
  di <- subset(d,Subject==index[i])  
  lines(di$age,di$distance,col=mycol[i])
}
for(k in 1:myk)
  abline(a=means[k,1],b=means[k,2],col=k+1,lwd=5)
legend("topleft",col=1:myk+1,lwd=5,
       legend=paste0("コンポーネント",1:myk))

# 7.6 4次元データに対する教師なし機械学習
x <- iris[,-5]
y <- as.numeric(iris$Species)
table(y)
pairs(x,col=y+1)
myk <- 3
set.seed(1)
res <- kmeans(x,centers=myk)
xtabs(~res$cluster+y)
# ランド指数
rand.index <- function(group1, group2){
  x <- abs(sapply(group1, function(x) x - group1))
  x[x > 1] <- 1
  y <- abs(sapply(group2, function(x) x - group2))
  y[y > 1] <- 1
  sg <- sum(abs(x - y))/2
  bc <- choose(dim(x)[1], 2)
  ri <- 1 - sg/bc
  return(ri)
}
rand.index(res$cluster,y)
# 混合正規分布による分類結果
myk <- 3
set.seed(1)
res <- densityMclust(x,G=myk,modelNames="VVV")
xtabs(~res$classification+y)
rand.index(res$classification,y)
# 階層型クラスター分析による分類結果
set.seed(1)
res <- hclust(dist(x),method="ward.D2")
myk <- 3
mycluster <- cutree(res,k=myk)
xtabs(~mycluster+y)
rand.index(mycluster,y)

# 7.7 高次元データに対する次元縮約と分類
# library(keras)
# install_keras() # kerasパッケージインストール後に1回だけ実行
# d0 <- dataset_mnist()
# d <- data.frame(d0$train$y,
#   array_reshape(d0$train$x,c(nrow(d0$train$x),28*28)))[1:2000,]
# colnames(d) <- c("label",paste0("pixel",0:783))
# write.csv(d,"mnist2000.csv",row.names=F)
# 保存データの読み込み
d <- read.csv("mnist2000.csv")
dim(d)
head(colnames(d))
tail(colnames(d))
y <- d[,1]
x <- d[,-1]
table(y)
myimage <- function(x){
  f <- matrix(data.matrix(x),28,28,byrow=T)
  image(t(f)[,28:1],col=gray.colors(255,rev=T))
}
par(mfrow=c(4,3),mar=c(2,2,1,1)) 
for(j in 0:9) myimage(x[which(y==j)[1],])

# 7.7.1 高次元データに対する主成分分析
res <- prcomp(x)
summary(res)$importance[,1:2]
myk <- 10
mycol <- 1:myk+1
plot(res$x[,1:2],type="n")
text(res$x[,1:2],labels=y,col=mycol[y+1],cex=1)

# 7.7.2 高次元データに対するt-SNE法
library(Rtsne)
set.seed(1)
res <- Rtsne(as.matrix(x))
mycol <- 1:myk+1
plot(res$Y,type="n")
text(res$Y,labels=y,col=mycol[y+1])

# 7.7.3 高次元データに対するUMAP法
tindex <- 1:1000
x.train <- x[tindex,]
x.test  <- x[-tindex,]
y.train <- y[tindex]
y.test  <- y[-tindex]
library(umap)
set.seed(1)
res <- umap(x.train)
library(RColorBrewer)
mycol <- brewer.pal(12,"Paired")
plot(res$layout,col=mycol[y.train+1],pch=19)
legend("topleft",legend=0:9,col=mycol,pch=19,cex=1.2)
# 階層型クラスター分析の適用
myk <- 10
mycluster <- cutree(
  hclust(dist(res$layout),method="ward.D2"),k=myk)
rand.index(mycluster,y.train)
# 検証データへの適用
res.test <- predict(res,x.test)
plot(res$layout,col=mycol[y.train+1],pch=19)
points(res.test,col=mycol[y.test+1],pch=17)
legend("topleft",legend=c("train","test"),pch=c(19,17))
myk <- 10
mycluster <- cutree(
  hclust(dist(res.test),method="ward.D2"),k=myk)
rand.index(mycluster,y.test)

# 7.8. 課題
# library(keras)
# install_keras() # kerasパッケージインストール後に1回だけ実行
# d0 <- dataset_fashion_mnist()
# d <- data.frame(d0$train$y,
#   array_reshape(d0$train$x,c(nrow(d0$train$x),28*28)))[1:2000,]
# colnames(d) <- c("label",paste0("pixel",0:783))
# write.csv(d,"fashion2000.csv",row.names=F)
# 保存データの読み込み
d <- read.csv("fashion2000.csv")
dim(d)
head(colnames(d))
tail(colnames(d))
y <- d[,1]
x <- d[,-1]
table(y)
par(mfrow=c(4,3),mar=c(2,2,1,1)) 
for(j in 0:9) myimage(x[which(y==j)[1],])
