# 8.1 判別データの読み込みと可視化
d <- read.csv("handwrite0.csv")
head(d)
f <- matrix(data.matrix(d$z),28,28,byrow=T)
image(t(f)[,28:1],col=gray.colors(255,rev=T))

(x <- matrix(1:9,ncol=3))
par(mfrow=c(1,2))            # 並べて描きたいのでグラフ画面を1行2列に分割
image(x,col=gray.colors(10,rev=T)) # 図8.2左
text(x=rep(c(0,0.5,1),c(3,3,3)),y=rep(c(0,0.5,1),3),labels=1:9,cex=2)
image(t(x)[,3:1],col=gray.colors(10,rev=T)) #転置と順番の入替:図8.2右
text(x=rep(c(0,0.5,1),3),y=rep(c(1,0.5,0),c(3,3,3)),labels=1:9,cex=2)
par(mfrow=c(1,1)) # グラフ画面を元に戻す

d$z <- ifelse(d$z>0,1,0) # d$z <- 1*(d$z>0)と書いてもよい
mycol <- ifelse(d$z > 0,"blue","gray")
plot(d[,1:2],col=mycol,cex=2,pch=22)

# 8.2 ロジスティック判別の適用
# 8.2.1 標準的なロジスティック回帰モデル
res <- glm(z~x+y,d,family="binomial")
summary(res)$coefficients
phat <- res$fitted.values
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat >= median(phat),1:2],col="red",cex=1.2,pch=15)

roc0 <- pROC:::roc(z~phat,d)
plot(roc0)

# 8.2.2 交互作用をもつロジスティック回帰
res <- glm(z~x*y, family="binomial", d)
summary(res)$coefficients
phat <- res$fitted.values
roc1 <- pROC:::roc(z~phat,d)  # roc1は後で利用するので保存しています
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat >= median(phat),1:2],col="red",cex=1.2,pch=15)

# 8.2.3 より高次の交互作用をもつロジスティック回帰
res <- glm(z~(1+x+I(x^2))*(1+y+I(y^2)),family="binomial",d)
summary(res)$coefficients
phat <- res$fitted.values
roc2 <- pROC:::roc(z~phat,d)   # roc2は後で利用するので保存しています
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat>=median(phat),1:2],col="red",cex=1.2,pch=15)

# 8.3 サポートベクターマシン (SVM)
library(kernlab)
res <- ksvm(z~x+y,d)
phat <- predict(res,d)
phat <- as.vector(phat)
roc3 <- pROC:::roc(z~phat,d)  # roc3は後で利用するので保存しています
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat>=0.5,1:2],col="red",cex=1.2,pch=15)

# 8.4 ニューラルネット
library(nnet)
set.seed(0)
res <- nnet(factor(z)~x+y,d,size=6) # size: 中間層のユニット数
set.seed(0)
res <- nnet(factor(z)~x+y,d,size=6,maxit=2000) # maxit=2000を追加
summary(res)

library(NeuralNetTools)
plotnet(res)
phat <- predict(res,d) # res$fitted.valuesでもよい
phat <- as.vector(phat)
roc4 <- pROC:::roc(z~phat,d)
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat>=0.5,1:2],col="red",cex=1.2,pch=15)

# 8.5 決定木 (分類木)
library(rpart)
res <- rpart(z~x+y,d,method="class")
plot(res)
text(res)
phat <- predict(res)[,2]
roc5 <- pROC:::roc(z~phat,d)
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat>0.5,1:2],col="red",cex=1.2,pch=15)

# 8.6 ランダムフォレスト
library(randomForest)
set.seed(0)
res <- randomForest(factor(z)~x+y,d)
phat <- predict(res,type="prob")[,2]
roc6 <- pROC:::roc(z~phat,d)
plot(d[,1:2],col=mycol,cex=2,pch=22)
points(d[phat>=0.5,1:2],col="red",cex=1.2,pch=15)

# 8.7 ROC曲線による判別性能の比較
plot(roc0, col="black")
plot(roc1, col="blue",  add=T)
plot(roc2, col="green", add=T)
plot(roc3, col="red",   add=T)
plot(roc4, col="orange",add=T)
plot(roc5, col="green4",add=T)
plot(roc6, col="red4",  add=T)
legend("bottomright",lty=rep(1,6),lwd=3,
col=c("black","blue","green","red","orange","green4","red4"),
legend=c("x+y","x*y","xx*yy","svm","nn","cart","rf"))
method=c("x+y","x*y","xx*yy","svm","nn","cart","rf")
auc=c(roc0$auc,roc1$auc,roc2$auc,roc3$auc,roc4$auc,roc5$auc,roc6$auc)
data.frame(method,auc)

# 8.8 多群分類データへの応用
# 8.8.1  多群分類データの読み込み
d <- read.csv("mnist2000.csv")
d$label <- as.factor(d$label)
head(d$label)
f <- matrix(data.matrix(d[3,-1]),28,28,byrow=T)
image(t(f)[,28:1],col=gray.colors(255,rev=T))

# 8.8.2 訓練データと検証データに分ける：予測の意味
m <- round(nrow(d)/2)
colnames(d)[1:6]
d.train <- d[1:m,]
d.test  <- d[(m+1):nrow(d),]

# 8.8.3 機械学習法における訓練データと検証データの利用例
# (a) サポートベクターマシン (SVM) における利用例
# SVM による予測モデルの構築
library(kernlab)
res <- ksvm(label~.,d.train)
c <- predict(res,newdata=d.train)
(f <- table(d.train[,1],c))
100*sum(diag(f))/nrow(d.train) 
#↑モデル構築の内部的な乱数の揺れのため書籍の数値と若干異なることが起こります

# SVM 予測モデルを検証データにあてはめる
c <- predict(res,newdata=d.test)
f <- table(d.test[,1],c)
print(f)
# 正判別率
100*sum(diag(f))/nrow(d.test)
#↑訓練データと同様に内部的な乱数の揺れのため書籍の数値と若干異なることが起こります

#（b）ランダムフォレスト (RF) における利用例
# RF による予測モデルの構築
library(randomForest)
set.seed(1)
res <- randomForest(label~., data=d.train)
varImpPlot(res) # importanceの高いピクセル
f <- matrix(data.matrix(res$importance),28,28,byrow=T)
image(t(f)[,28:1])

# RF 予測モデルを検証データにあてはめる
# 予測モデルの訓練データへのあてはめ
c <- predict(res, newdata=d.train, type="class")
(f <- table(d.train[,1],c))
100*sum(diag(f))/nrow(d.train)
#  予測モデルの検証データへのあてはめ
c <- predict(res, newdata=d.test, type="class")
(f <- table(d.test[,1],c))
100*sum(diag(f))/nrow(d.test)

# 8.9 課題
d <- read.csv("fashion2000.csv")
d$label <- as.factor(d$label)
