# プレセミナー第４回
# R言語でグラフを描こう

# P4.1. plot関数によるグラフの作成

# plot関数
plot(iris$Sepal.Length,iris$Sepal.Width)
 
# タイトルを追加する
plot(iris$Sepal.Length,iris$Sepal.Width, xlab="がくの長さ", ylab="がくの幅", main="irisデータ")

 
# データ点ごとに形状を指定する
shape <- rep(0,150)
shape[iris$Species=="setosa"] <- 1
shape[iris$Species=="versicolor"] <- 2
shape[iris$Species=="virginica"] <- 4

plot(iris$Sepal.Length,iris$Sepal.Width, xlab="がくの長さ", ylab="がくの幅", main="irisデータ",pch=shape)
 
# レジェンドを追加する
legend("topright", legend = c("setosa","versicolor","virginica"), pch = c(1,2,4))
 
# 直線を上書きする
abline(-2.8,1.1)

abline(v=7.1, col="blue")
abline(h=3.5, col="red")
 
# 折れ線グラフをプロットする
x <- 1:12
y <- sin(x)
plot(x,y,type="l")

# 折れ線グラフの重ね描き
y2 <- cos(x)
points(x, y2, type="l", col="red")
 
# 表示範囲を指定する
plot(x,y,type="p",xlim=c(6,10),ylim=c(-0.5,1))
 
# データフレームに対してplot関数を用いると、散布図行列が表示される
plot(iris)


# P4.2. グラフの保存
# png関数
png("iris.png")
plot(iris)
dev.off()

# サイズと解像度を指定する
png("iris2.png", width=960, height=960, res=144)
plot(iris)
dev.off()

# pdf関数
pdf("iris.pdf")
plot(iris)
dev.off()

# 複数データの保存
pdf("iris2.pdf")
plot(iris)
boxplot(iris)
dev.off()

# 日本語が含まれる図の保存
pdf("iris_japan.pdf",family = "Japan1")
plot(iris$Sepal.Length,iris$Sepal.Width, xlab="がくの長さ", ylab="がくの幅", main="irisデータ")
dev.off()


# P4.3. par関数による高度な設定
par()

# グラフを縦に2個、横に3個表示する
par(mfrow=c(2,3))

x <- 1:12
y <- sin(x)
for(i in 1:6) plot(x,y,pch=i,cex=1.2)

# プロット間の間隔の調整
par(mar=c(4,4,1,1))
for(i in 1:6) plot(x,y,pch=i,cex=1.2)

# dev.offを用いたパラメータの初期化
dev.off()
for(i in 1:6) plot(x,y,pch=i,cex=1.2)

# P4.4. その他のグラフ

# P4.4.1 棒グラフ
barplot(c(2,4,8,16))

# 棒グラフにラベルを追加
barplot(c(2,4,8,16), names=c("A", "B", "C", "D"))

# P4.4.2 ヒストグラム
hist(iris$Sepal.Length)

# ヒストグラムの区間を調整
hist(iris$Sepal.Length, breaks=c(4, 6, 8))
hist(iris$Sepal.Length, breaks=seq(4, 8, 0.1))


# P4.4.3 画像の描画
# 数値行列を可視化
X <- matrix(1:20, ncol=5)
X

image(X)
text(x=rep(c(1,0.667,0.333,0),3),y=rep(c(1,0.75,0.5,0.25,0),c(4,4,4,4,4)),labels=20:1)
image(X,col=gray.colors(20))

# P4.4.4 ヒートマップ
mt1 <- as.matrix(iris[,-5])
image(t(mt1))
heatmap(mt1)
heatmap(mt1,Colv=NA,Rowv=NA)
heatmap(mt1,Colv=NA,Rowv=NA,scale="row")
heatmap(mt1,Colv=NA,Rowv=NA,scale="none")
heatmap(mt1,Colv=NA,Rowv=NA,scale="column")
